package sun.util.calendar;

import java.util.Locale;
import java.util.TimeZone;

public abstract class CalendarDate implements Cloneable
{
    public static final int FIELD_UNDEFINED = Integer.MIN_VALUE;
    public static final long TIME_UNDEFINED = Long.MIN_VALUE;
    private Era era;
    private int year;
    private int month;
    private int dayOfMonth;
    private int dayOfWeek;
    private boolean leapYear;
    private int hours;
    private int minutes;
    private int seconds;
    private int millis;
    private long fraction;
    private boolean normalized;
    private TimeZone zoneinfo;
    private int zoneOffset;
    private int daylightSaving;
    private boolean forceStandardTime;
    private Locale locale;
    
    protected CalendarDate() {
        this(TimeZone.getDefault());
    }
    
    protected CalendarDate(TimeZone zoneinfo) {
        this.dayOfWeek = Integer.MIN_VALUE;
        this.zoneinfo = zoneinfo;
    }
    
    public Era getEra() {
        return this.era;
    }
    
    public CalendarDate setEra(Era era) {
        if (this.era == era) {
            return this;
        }
        this.era = era;
        this.normalized = false;
        return this;
    }
    
    public int getYear() {
        return this.year;
    }
    
    public CalendarDate setYear(int year) {
        if (this.year != year) {
            this.year = year;
            this.normalized = false;
            return this;
        }
        return this;
    }
    
    public CalendarDate addYear(int n) {
        if (n != 0) {
            this.year += n;
            this.normalized = false;
            return this;
        }
        return this;
    }
    
    public boolean isLeapYear() {
        return this.leapYear;
    }
    
    void setLeapYear(boolean leapYear) {
        this.leapYear = leapYear;
    }
    
    public int getMonth() {
        return this.month;
    }
    
    public CalendarDate setMonth(int month) {
        if (this.month != month) {
            this.month = month;
            this.normalized = false;
            return this;
        }
        return this;
    }
    
    public CalendarDate addMonth(int n) {
        if (n != 0) {
            this.month += n;
            this.normalized = false;
            return this;
        }
        return this;
    }
    
    public int getDayOfMonth() {
        return this.dayOfMonth;
    }
    
    public CalendarDate setDayOfMonth(int dayOfMonth) {
        if (this.dayOfMonth != dayOfMonth) {
            this.dayOfMonth = dayOfMonth;
            this.normalized = false;
            return this;
        }
        return this;
    }
    
    public CalendarDate addDayOfMonth(int n) {
        if (n != 0) {
            this.dayOfMonth += n;
            this.normalized = false;
            return this;
        }
        return this;
    }
    
    public int getDayOfWeek() {
        if (!this.isNormalized()) {
            this.dayOfWeek = Integer.MIN_VALUE;
        }
        return this.dayOfWeek;
    }
    
    public int getHours() {
        return this.hours;
    }
    
    public CalendarDate setHours(int hours) {
        if (this.hours != hours) {
            this.hours = hours;
            this.normalized = false;
            return this;
        }
        return this;
    }
    
    public CalendarDate addHours(int n) {
        if (n != 0) {
            this.hours += n;
            this.normalized = false;
            return this;
        }
        return this;
    }
    
    public int getMinutes() {
        return this.minutes;
    }
    
    public CalendarDate setMinutes(int minutes) {
        if (this.minutes != minutes) {
            this.minutes = minutes;
            this.normalized = false;
            return this;
        }
        return this;
    }
    
    public CalendarDate addMinutes(int n) {
        if (n != 0) {
            this.minutes += n;
            this.normalized = false;
            return this;
        }
        return this;
    }
    
    public int getSeconds() {
        return this.seconds;
    }
    
    public CalendarDate setSeconds(int seconds) {
        if (this.seconds != seconds) {
            this.seconds = seconds;
            this.normalized = false;
            return this;
        }
        return this;
    }
    
    public CalendarDate addSeconds(int n) {
        if (n != 0) {
            this.seconds += n;
            this.normalized = false;
            return this;
        }
        return this;
    }
    
    public int getMillis() {
        return this.millis;
    }
    
    public CalendarDate setMillis(int millis) {
        if (this.millis != millis) {
            this.millis = millis;
            this.normalized = false;
            return this;
        }
        return this;
    }
    
    public CalendarDate addMillis(int n) {
        if (n != 0) {
            this.millis += n;
            this.normalized = false;
            return this;
        }
        return this;
    }
    
    public long getTimeOfDay() {
        if (!this.isNormalized()) {
            return this.fraction = Long.MIN_VALUE;
        }
        return this.fraction;
    }
    
    public CalendarDate setDate(int year, int month, int dayOfMonth) {
        this.setYear(year);
        this.setMonth(month);
        this.setDayOfMonth(dayOfMonth);
        return this;
    }
    
    public CalendarDate addDate(int n, int n2, int n3) {
        this.addYear(n);
        this.addMonth(n2);
        this.addDayOfMonth(n3);
        return this;
    }
    
    public CalendarDate setTimeOfDay(int hours, int minutes, int seconds, int millis) {
        this.setHours(hours);
        this.setMinutes(minutes);
        this.setSeconds(seconds);
        this.setMillis(millis);
        return this;
    }
    
    public CalendarDate addTimeOfDay(int n, int n2, int n3, int n4) {
        this.addHours(n);
        this.addMinutes(n2);
        this.addSeconds(n3);
        this.addMillis(n4);
        return this;
    }
    
    protected void setTimeOfDay(long fraction) {
        this.fraction = fraction;
    }
    
    public boolean isNormalized() {
        return this.normalized;
    }
    
    public boolean isStandardTime() {
        return this.forceStandardTime;
    }
    
    public void setStandardTime(boolean forceStandardTime) {
        this.forceStandardTime = forceStandardTime;
    }
    
    public boolean isDaylightTime() {
        return !this.isStandardTime() && this.daylightSaving != 0;
    }
    
    protected void setLocale(Locale locale) {
        this.locale = locale;
    }
    
    public TimeZone getZone() {
        return this.zoneinfo;
    }
    
    public CalendarDate setZone(TimeZone zoneinfo) {
        this.zoneinfo = zoneinfo;
        return this;
    }
    
    public boolean isSameDate(CalendarDate calendarDate) {
        return this.getDayOfWeek() == calendarDate.getDayOfWeek() && this.getMonth() == calendarDate.getMonth() && this.getYear() == calendarDate.getYear() && this.getEra() == calendarDate.getEra();
    }
    
    @Override
    public boolean equals(Object o) {
        if (!(o instanceof CalendarDate)) {
            return false;
        }
        CalendarDate calendarDate = (CalendarDate)o;
        if (this.isNormalized() != calendarDate.isNormalized()) {
            return false;
        }
        int n = this.zoneinfo != null;
        return n == ((calendarDate.zoneinfo != null) ? 1 : 0) && (n == 0 || this.zoneinfo.equals(calendarDate.zoneinfo)) && this.getEra() == calendarDate.getEra() && this.year == calendarDate.year && this.month == calendarDate.month && this.dayOfMonth == calendarDate.dayOfMonth && this.hours == calendarDate.hours && this.minutes == calendarDate.minutes && this.seconds == calendarDate.seconds && this.millis == calendarDate.millis && this.zoneOffset == calendarDate.zoneOffset;
    }
    
    @Override
    public int hashCode() {
        long n = (((((((long)this.year - 1970L) * 12L + (long)(this.month - 1)) * 30L + (long)this.dayOfMonth) * 24L + (long)this.hours) * 60L + (long)this.minutes) * 60L + (long)this.seconds) * 1000L + (long)this.millis - (long)this.zoneOffset;
        int normalized = this.isNormalized();
        int hashCode = 0;
        Era era = this.getEra();
        if (era != null) {
            hashCode = era.hashCode();
        }
        return (int)n * (int)(n >> 32) ^ hashCode ^ normalized ^ ((this.zoneinfo != null) ? this.zoneinfo.hashCode() : 0);
    }
    
    public Object clone() {
        try {
            return super.clone();
        }
        catch (CloneNotSupportedException cause) {
            throw new InternalError(cause);
        }
    }
    
    @Override
    public String toString() {
        StringBuilder sb = new StringBuilder();
        CalendarUtils.sprintf0d(sb, this.year, 4).append('-');
        CalendarUtils.sprintf0d(sb, this.month, 2).append('-');
        CalendarUtils.sprintf0d(sb, this.dayOfMonth, 2).append('T');
        CalendarUtils.sprintf0d(sb, this.hours, 2).append(':');
        CalendarUtils.sprintf0d(sb, this.minutes, 2).append(':');
        CalendarUtils.sprintf0d(sb, this.seconds, 2).append('.');
        CalendarUtils.sprintf0d(sb, this.millis, 3);
        if (this.zoneOffset == 0) {
            sb.append('Z');
        }
        else if (this.zoneOffset != Integer.MIN_VALUE) {
            int zoneOffset;
            int c;
            if (this.zoneOffset > 0) {
                zoneOffset = this.zoneOffset;
                c = '+';
            }
            else {
                zoneOffset = -this.zoneOffset;
                c = '-';
            }
            int n = zoneOffset / 60000;
            sb.append((char)c);
            CalendarUtils.sprintf0d(sb, n / 60, 2);
            CalendarUtils.sprintf0d(sb, n % 60, 2);
        }
        else {
            sb.append(" local time");
        }
        return sb.toString();
    }
    
    protected void setDayOfWeek(int dayOfWeek) {
        this.dayOfWeek = dayOfWeek;
    }
    
    protected void setNormalized(boolean normalized) {
        this.normalized = normalized;
    }
    
    public int getZoneOffset() {
        return this.zoneOffset;
    }
    
    protected void setZoneOffset(int zoneOffset) {
        this.zoneOffset = zoneOffset;
    }
    
    public int getDaylightSaving() {
        return this.daylightSaving;
    }
    
    protected void setDaylightSaving(int daylightSaving) {
        this.daylightSaving = daylightSaving;
    }
}
